@@ -1,3 +1,196 @@
+//! # Skytable client [![Crates.io](https://img.shields.io/crates/v/skytable?style=flat-square)](https://crates.io/crates/skytable) [![Test](https://github.com/skytable/client-rust/actions/workflows/test.yml/badge.svg)](https://github.com/skytable/client-rust/actions/workflows/test.yml) [![docs.rs](https://img.shields.io/docsrs/skytable?style=flat-square)](https://docs.rs/skytable) [![GitHub release (latest SemVer including pre-releases)](https://img.shields.io/github/v/release/skytable/client-rust?include_prereleases&style=flat-square)](https://github.com/skytable/client-rust/releases)
+//! 
+//! ## Introduction
+//! 
+//! This library is the official client for the free and open-source NoSQL database
+//! [Skytable](https://github.com/skytable/skytable). First, go ahead and install Skytable by
+//! following the instructions [here](https://docs.skytable.io/getting-started). This library supports
+//! all Skytable versions that work with the [Skyhash 1.1 Protocol](https://docs.skytable.io/protocol/skyhash).
+//! This version of the library was tested with the latest Skytable release
+//! (release [0.7.2](https://github.com/skytable/skytable/releases/v0.7.2)).
+//! 
+//! ## Features
+//! 
+//! - Sync API
+//! - Async API
+//! - TLS in both sync/async APIs
+//! - Connection pooling for sync/async
+//! - Use both sync/async APIs at the same time
+//! - Always up-to-date
+//! 
+//! ## Using this library
+//! 
+//! This library only ships with the bare minimum that is required for interacting with Skytable. Once you have
+//! Skytable installed and running, you're ready to follow this guide!
+//! 
+//! We'll start by creating a new binary application and then running actions. Create a new binary application
+//! by running:
+//! 
+//! ```shell
+//! cargo new skyapp
+//! ```
+//! 
+//! **Tip**: You can see a full list of the available actions [here](https://docs.skytable.io/actions-overview).
+//! 
+//! First add this to your `Cargo.toml` file:
+//! 
+//! ```toml
+//! skytable = "0.6.3"
+//! ```
+//! 
+//! Now open up your `src/main.rs` file and establish a connection to the server while also adding some
+//! imports:
+//! 
+//! ```norun
+//! use skytable::{Connection, Query, Element};
+//! fn main() -> std::io::Result<()> {
+//!     let mut con = Connection::new("127.0.0.1", 2003)?;
+//!     Ok(())
+//! }
+//! ```
+//! 
+//! Now let's run a `Query`! Change the previous code block to:
+//! 
+//! ```norun
+//! use skytable::{error, Connection, Query, Element};
+//! fn main() -> Result<(), error::Error> {
+//!     let mut con = Connection::new("127.0.0.1", 2003)?;
+//!     let query = Query::from("heya");
+//!     let res = con.run_simple_query(&query)?;
+//!     assert_eq!(res, Element::String("HEY".to_owned()));
+//!     Ok(())
+//! }
+//! ```
+//! 
+//! ## Running actions
+//! 
+//! As noted [below](#binary-data), the default table is a key/value table with a binary key
+//! type and a binary value type. Let's go ahead and run some actions (we're assuming you're
+//! using the sync API; for async, simply change the import to `use skytable::actions::AsyncActions`).
+//! 
+//! ### `SET`ting a key
+//! 
+//! ```norun
+//! use skytable::actions::Actions;
+//! use skytable::sync::Connection;
+//! 
+//! let mut con = Connection::new("127.0.0.1", 2003).unwrap();
+//! con.set("hello", "world").unwrap();
+//! ```
+//! 
+//! This will set the value of the key `hello` to `world` in the `default:default` entity.
+//! 
+//! ### `GET`ting a key
+//! 
+//! ```norun
+//! use skytable::actions::Actions;
+//! use skytable::sync::Connection;
+//! 
+//! let mut con = Connection::new("127.0.0.1", 2003).unwrap();
+//! let x: String = con.get("hello").unwrap();
+//! assert_eq!(x, "world");
+//! ```
+//! 
+//! Way to go &mdash; you're all set! Now go ahead and run more advanced queries!
+//! 
+//! ## Binary data
+//! 
+//! The `default:default` keyspace has the following declaration:
+//! 
+//! ```text
+//! Keymap { data:(binstr,binstr), volatile:false }
+//! ```
+//! 
+//! This means that the default keyspace is ready to store binary data. Let's say
+//! you wanted to `SET` the value of a key called `bindata` to some binary data stored
+//! in a `Vec<u8>`. You can achieve this with the `RawString` type:
+//! 
+//! ```norun
+//! use skytable::actions::Actions;
+//! use skytable::sync::Connection;
+//! use skytable::types::RawString;
+//! 
+//! let mut con = Connection::new("127.0.0.1", 2003).unwrap();
+//! let mybinarydata = RawString::from(vec![1, 2, 3, 4]);
+//! assert!(con.set("bindata", mybinarydata).unwrap());
+//! ```
+//! 
+//! ## Going advanced
+//! 
+//! Now that you know how you can run basic queries, check out the [`actions`] module documentation for learning
+//! to use actions and the [`types`] module documentation for implementing your own Skyhash serializable
+//! types. Need to meddle with DDL queries like creating and dropping tables? Check out the [`ddl`] module.
+//! You can also find some [examples here](https://github.com/skytable/client-rust/tree/v0.7.0-alpha.1/examples)
+//! 
+//! ## Connection pooling
+//! 
+//! This library supports using sync/async connection pools. See the [`pool`] module-level documentation for examples
+//! and information.
+//! 
+//! ## Async API
+//! 
+//! If you need to use an `async` API, just change your import to:
+//! 
+//! ```toml
+//! skytable = { version = "0.6.3", features=["aio"], default-features = false }
+//! ```
+//! 
+//! You can now establish a connection by using `skytable::AsyncConnection::new()`, adding `.await`s wherever
+//! necessary. Do note that you'll the [Tokio runtime](https://tokio.rs).
+//! 
+//! ## Using both `sync` and `async` APIs
+//! 
+//! With this client driver, it is possible to use both sync and `async` APIs **at the same time**. To do
+//! this, simply change your import to:
+//! 
+//! ```toml
+//! skytable = { version="0.6.3", features=["sync", "aio"] }
+//! ```
+//! 
+//! ## TLS
+//! 
+//! If you need to use TLS features, this crate will let you do so with OpenSSL.
+//! 
+//! ### Using TLS with sync interfaces
+//! 
+//! ```toml
+//! skytable = { version="0.6.3", features=["sync","ssl"] }
+//! ```
+//! 
+//! You can now use the async `sync::TlsConnection` object.
+//! 
+//! ### Using TLS with async interfaces
+//! 
+//! ```toml
+//! skytable = { version="0.6.3", features=["aio","aio-ssl"], default-features=false }
+//! ```
+//! 
+//! You can now use the async `aio::TlsConnection` object.
+//! 
+//! ### _Packed TLS_ setup
+//! 
+//! If you want to pack OpenSSL with your crate, then for sync add `sslv` instead of `ssl` or
+//! add `aio-sslv` instead of `aio-ssl` for async. Adding this will statically link OpenSSL
+//! to your crate. Do note that you'll need a C compiler, GNU Make and Perl to compile OpenSSL
+//! and statically link against it.
+//! 
+//! ## MSRV
+//! 
+//! The MSRV for this crate is Rust 1.39. Need const generics? Add the `const-gen` feature to your
+//! dependency!
+//! 
+//! ## Contributing
+//! 
+//! Open-source, and contributions ... &mdash; they're always welcome! For ideas and suggestions,
+//! [create an issue on GitHub](https://github.com/skytable/client-rust/issues/new) and for patches,
+//! fork and open those pull requests [here](https://github.com/skytable/client-rust)!
+//! 
+//! ## License
+//! 
+//! This client library is distributed under the permissive
+//! [Apache-2.0 License](https://github.com/skytable/client-rust/blob/next/LICENSE). Now go build great apps!
+//!
+
 /*
  * Created on Wed May 05 2021
  *
@@ -15,163 +208,6 @@
  *
 */
 
-//! # Skytable client
-//!
-//! This library is the official client for the free and open-source NoSQL database
-//! [Skytable](https://github.com/skytable/skytable). First, go ahead and install Skytable by
-//! following the instructions [here](https://docs.skytable.io/getting-started). This library supports
-//! all Skytable versions that work with the [Skyhash 1.1 Protocol](https://docs.skytable.io/protocol/skyhash).
-//! This version of the library was tested with the latest Skytable release
-//! (release [0.7](https://github.com/skytable/skytable/releases/v0.7.0)).
-//!
-//! ## Using this library
-//!
-//! This library only ships with the bare minimum that is required for interacting with Skytable. Once you have
-//! Skytable installed and running, you're ready to follow this guide!
-//!
-//! We'll start by creating a new binary application and then running actions. Create a new binary application
-//! by running:
-//! ```shell
-//! cargo new skyapp
-//! ```
-//! **Tip**: You can see a full list of the available actions [here](https://docs.skytable.io/actions-overview).
-//!
-//! First add this to your `Cargo.toml` file:
-//! ```toml
-//! skytable = "0.6.2"
-//! ```
-//! Now open up your `src/main.rs` file and establish a connection to the server while also adding some
-//! imports:
-//! ```no_run
-//! use skytable::{Connection, Query, Element};
-//! fn main() -> std::io::Result<()> {
-//!     let mut con = Connection::new("127.0.0.1", 2003)?;
-//!     Ok(())
-//! }
-//! ```
-//!
-//! Now let's run a [`Query`]! Change the previous code block to:
-//! ```no_run
-//! use skytable::{error, Connection, Query, Element};
-//! fn main() -> Result<(), error::Error> {
-//!     let mut con = Connection::new("127.0.0.1", 2003)?;
-//!     let query = Query::from("heya");
-//!     let res = con.run_simple_query(&query)?;
-//!     assert_eq!(res, Element::String("HEY".to_owned()));
-//!     Ok(())
-//! }
-//! ```
-//!
-//! ## Running actions
-//!
-//! As noted [below](#binary-data), the default table is a key/value table with a binary key
-//! type and a binary value type. Let's go ahead and run some actions (we're assuming you're
-//! using the sync API; for async, simply change the import to `use skytable::actions::AsyncActions`).
-//!
-//! ### `SET`ting a key
-//!
-//! ```no_run
-//! use skytable::actions::Actions;
-//! use skytable::sync::Connection;
-//!
-//! let mut con = Connection::new("127.0.0.1", 2003).unwrap();
-//! con.set("hello", "world").unwrap();
-//! ```
-//!
-//! This will set the value of the key `hello` to `world` in the `default:default` entity.
-//!
-//! ### `GET`ting a key
-//!
-//! ```no_run
-//! use skytable::actions::Actions;
-//! use skytable::sync::Connection;
-//!
-//! let mut con = Connection::new("127.0.0.1", 2003).unwrap();
-//! let x: String = con.get("hello").unwrap();
-//! assert_eq!(x, "world");
-//! ```
-//!
-//! Way to go &mdash; you're all set! Now go ahead and run more advanced queries!
-//!
-//! ## Binary data
-//!
-//! The `default:default` keyspace has the following declaration:
-//! ```json
-//! Keymap { data:(binstr,binstr), volatile:false }
-//! ```
-//!
-//! This means that the default keyspace is ready to store binary data. Let's say
-//! you wanted to `SET` the value of a key called `bindata` to some binary data stored
-//! in a `Vec<u8>`. You can achieve this with the [`RawString`](types::RawString) type:
-//!
-//! ```no_run
-//! use skytable::actions::Actions;
-//! use skytable::sync::Connection;
-//! use skytable::types::RawString;
-//!
-//! let mut con = Connection::new("127.0.0.1", 2003).unwrap();
-//! let mybinarydata = RawString::from(vec![1, 2, 3, 4]);
-//! assert!(con.set("bindata", mybinarydata).unwrap());
-//! ```
-//!
-//! ## Going advanced
-//!
-//! Now that you know how you can run basic queries, check out the [`actions`] module documentation for learning
-//! to use actions and the [`types`] module documentation for implementing your own Skyhash serializable
-//! types. Need to meddle with DDL queries like creating and dropping tables? Check out the [`ddl`] module.
-//! You can also find the [latest examples here](https://github.com/skytable/client-rust/tree/next/examples)
-//!
-//! ## Async API
-//!
-//! If you need to use an `async` API, just change your import to:
-//! ```toml
-//! skytable = { version = "0.6.2", features=["async"], default-features=false }
-//! ```
-//! You can now establish a connection by using `skytable::AsyncConnection::new()`, adding `.await`s wherever
-//! necessary. Do note that you'll the [Tokio runtime](https://tokio.rs).
-//!
-//! ## Using both `sync` and `async` APIs
-//!
-//! With this client driver, it is possible to use both sync and `async` APIs **at the same time**. To do
-//! this, simply change your import to:
-//! ```toml
-//! skytable = { version="0.6.2", features=["sync", "async"] }
-//! ```
-//!
-//! ## TLS
-//!
-//! If you need to use TLS features, this crate will let you do so with OpenSSL.
-//!
-//! ### Using TLS with sync interfaces
-//! ```toml
-//! skytable = { version="0.6.2", features=["sync","ssl"] }
-//! ```
-//! You can now use the async [TlsConnection](`sync::TlsConnection`) object.
-//!
-//! ### Using TLS with async interfaces
-//! ```toml
-//! skytable = { version="0.6.2", features=["async","aio-ssl"], default-features=false }
-//! ```
-//! You can now use the async [TlsConnection](`aio::TlsConnection`) object.
-//!
-//! ### _Packed TLS_ setup
-//!
-//! If you want to pack OpenSSL with your crate, then for sync add `sslv` instead of `ssl` or
-//! add `aio-sslv` instead of `aio-ssl` for async. Adding this will statically link OpenSSL
-//! to your crate. Do note that you'll need a C compiler, GNU Make and Perl to compile OpenSSL
-//! and statically link against it.
-//!
-//! ## Contributing
-//!
-//! Open-source, and contributions ... &mdash; they're always welcome! For ideas and suggestions,
-//! [create an issue on GitHub](https://github.com/skytable/client-rust/issues/new) and for patches,
-//! fork and open those pull requests [here](https://github.com/skytable/client-rust)!
-//!
-//! ## License
-//! This client library is distributed under the permissive
-//! [Apache-2.0 License](https://github.com/skytable/client-rust/blob/next/LICENSE). Now go build great apps!
-//!
-
 #![cfg_attr(docsrs, feature(doc_cfg))]
 // macro mods
 #[macro_use]
@@ -181,6 +217,13 @@ mod util;
 pub mod actions;
 pub mod ddl;
 pub mod error;
+#[cfg(any(
+    feature = "sync",
+    feature = "pool",
+    feature = "aio",
+    feature = "aio-pool"
+))]
+pub mod pool;
 pub mod types;
 // endof public mods
 // private mods
@@ -202,21 +245,24 @@ pub const DEFAULT_PORT: u16 = 2003;
 pub const DEFAULT_ENTITY: &str = "default:default";
 
 cfg_async!(
+    use core::{future::Future, pin::Pin};
     pub mod aio;
     pub use aio::Connection as AsyncConnection;
     use tokio::io::AsyncWriteExt;
+    /// A special result that is returned when running actions (async)
+    pub type AsyncResult<'s, T> = Pin<Box<dyn Future<Output = T> + Send + Sync + 's>>;
 );
+
 cfg_sync!(
     pub mod sync;
     pub use sync::Connection;
 );
 
 /// A generic result type
-pub type SkyRawResult<T> = Result<T, self::error::Error>;
-/// A specialized error type for queries
-pub type SkyResult = SkyRawResult<Element>;
+pub type SkyResult<T> = Result<T, self::error::Error>;
+/// A result type for queries
+pub type SkyQueryResult = SkyResult<Element>;
 
-#[derive(Debug, Clone)]
 /// A connection builder for easily building connections
 ///
 /// ## Example (sync)
@@ -245,10 +291,11 @@ pub type SkyResult = SkyRawResult<Element>;
 ///         .unwrap();
 /// }
 /// ```
+#[derive(Debug, Clone)]
 pub struct ConnectionBuilder {
-    port: Option<u16>,
-    host: Option<String>,
-    entity: Option<String>,
+    port: u16,
+    host: String,
+    entity: String,
 }
 
 impl Default for ConnectionBuilder {
@@ -260,34 +307,30 @@ impl Default for ConnectionBuilder {
 impl ConnectionBuilder {
     /// Create an empty connection builder
     pub fn new() -> Self {
-        Self {
-            port: None,
-            host: None,
-            entity: None,
-        }
+        Self::default()
     }
     /// Set the port (defaults to `2003`)
     pub fn set_port(mut self, port: u16) -> Self {
-        self.port = Some(port);
+        self.port = port;
         self
     }
     /// Set the host (defaults to `localhost`)
     pub fn set_host(mut self, host: String) -> Self {
-        self.host = Some(host);
+        self.host = host;
         self
     }
     /// Set the entity (defaults to `default:default`)
     pub fn set_entity(mut self, entity: String) -> Self {
-        self.entity = Some(entity);
+        self.entity = entity;
         self
     }
     cfg_sync! {
         /// Get a [sync connection](sync::Connection) to the database
-        pub fn get_connection(&self) -> SkyRawResult<sync::Connection> {
+        pub fn get_connection(&self) -> SkyResult<sync::Connection> {
             use crate::ddl::Ddl;
             let mut con =
-                sync::Connection::new(self.host.as_ref().unwrap_or(&DEFAULT_HOSTADDR.to_owned()), self.port.unwrap_or(2003))?;
-            con.switch(self.entity.as_ref().unwrap_or(&DEFAULT_ENTITY.to_owned()))?;
+                sync::Connection::new(&self.host, self.port)?;
+            con.switch(&self.entity)?;
             Ok(con)
         }
         cfg_sync_ssl_any! {
@@ -295,25 +338,25 @@ impl ConnectionBuilder {
             pub fn get_tls_connection(
                 &self,
                 sslcert: String,
-            ) -> SkyRawResult<sync::TlsConnection> {
+            ) -> SkyResult<sync::TlsConnection> {
                 use crate::ddl::Ddl;
                 let mut con = sync::TlsConnection::new(
-                    self.host.as_ref().unwrap_or(&DEFAULT_HOSTADDR.to_owned()),
-                    self.port.unwrap_or(2003),
+                    &self.host,
+                    self.port,
                     &sslcert,
                 )?;
-                con.switch(self.entity.as_ref().unwrap_or(&DEFAULT_ENTITY.to_owned()))?;
+                con.switch(&self.entity)?;
                 Ok(con)
             }
         }
     }
     cfg_async! {
         /// Get an [async connection](aio::Connection) to the database
-        pub async fn get_async_connection(&self) -> SkyRawResult<aio::Connection> {
+        pub async fn get_async_connection(&self) -> SkyResult<aio::Connection> {
             use crate::ddl::AsyncDdl;
-            let mut con = aio::Connection::new(self.host.as_ref().unwrap_or(&DEFAULT_HOSTADDR.to_owned()), self.port.unwrap_or(2003))
+            let mut con = aio::Connection::new(&self.host, self.port)
                 .await?;
-            con.switch(self.entity.as_ref().unwrap_or(&DEFAULT_ENTITY.to_owned())).await?;
+            con.switch(&self.entity).await?;
             Ok(con)
         }
         cfg_async_ssl_any! {
@@ -321,15 +364,15 @@ impl ConnectionBuilder {
             pub async fn get_async_tls_connection(
                 &self,
                 sslcert: String,
-            ) -> SkyRawResult<aio::TlsConnection> {
+            ) -> SkyResult<aio::TlsConnection> {
                 use crate::ddl::AsyncDdl;
                 let mut con = aio::TlsConnection::new(
-                    self.host.as_ref().unwrap_or(&DEFAULT_HOSTADDR.to_owned()),
-                    self.port.unwrap_or(2003),
+                    &self.host,
+                    self.port,
                     &sslcert,
                 )
                 .await?;
-                con.switch(self.entity.as_ref().unwrap_or(&DEFAULT_ENTITY.to_owned())).await?;
+                con.switch(&self.entity).await?;
                 Ok(con)
             }
         }
@@ -368,8 +411,6 @@ cfg_sync! {
 }
 
 cfg_async! {
-    use core::pin::Pin;
-    use core::future::Future;
     use tokio::io::AsyncWrite;
     type FutureRet<'s> = Pin<Box<dyn Future<Output = IoResult<()>> + Send + Sync + 's>>;
     trait WriteQueryAsync<T: AsyncWrite + Unpin + Send + Sync>: Unpin + Sync + Send {
